//
//  PlayList.cpp
//  Session 10
//
//  Created by Luis Angel on 6/4/15.
//  Copyright (c) 2015 Luis Angel. All rights reserved.
//

#include <iostream>
#include <string>

using namespace std;

////////////////////////////////////// Class declarations //////////////////////////////////////////

const int MAX_SONGS = 100;

class PlayList;

class Song
{
private:
	string title;				// Song's title.
	string genre;				// Song's genre.
	string singer;				// Song's singer.
	PlayList* playListPtr;		// Pointer to its playlist.
	
public:
	// Constructor.
	// Parameters: t -> title, g -> genre, s -> singer, ptr -> playListPtr.
	Song( string t, string g, string s, PlayList* ptr );
	
	// Accessors to private members.
	string getTitle() const;
	string getGenre() const;
	string getSinger() const;
	
	// Display (cout) the all songs in its playlist that match
	// "this" song's << genre >>.
	void getSimilarSongs() const;
	
};

class PlayList
{
private:
	int nSongs;					// Effective number of songs.
	Song* songs[MAX_SONGS];		// An array of up to MAX_SONGS dynamically
								// allocated Song objects.
public:
	// Constructor.
	// Just initialize nSongs to 0.
	PlayList();
	
	// Destructor.
	// Delete all remaining Song objects pointed to by the array songs[].
	~PlayList();
	
	// Add a song.
	// Allocate a new dynamic song with the given properties. Upon success (e.g. there
	// still space in songs[]) return a pointer to the (constant) newly created Song
	// object; otherwise, return nullptr.
	const Song* addSong( const string title, const string genre, const string singer );
	
	// Utility.
	// Populate the list[] array with pointers to songs that have the given genre.
	// Return the number of Song pointers you inserted in list[].
	int getSongsByGenre( const string genre, Song* list[] ) const;
	
	// Removing all songs that correspond to some artist.
	// Delete the dynamically created Song objects whose artist is singer.
	// Return the number of removed songs.
	int removeSongsBySinger( const string singer );
};

////////////////////////////////////// Class definitions for Song //////////////////////////////////

// Constructor.
Song::Song( string t, string g, string s, PlayList* ptr )
{
	title = t;
	genre = g;
	singer = s;
	playListPtr = ptr;
}

string Song::getTitle() const
{
	return title;
}

string Song::getGenre() const
{
	return genre;
}

string Song::getSinger() const
{
	return singer;
}

void Song::getSimilarSongs() const
{
	Song* similarSongs[MAX_SONGS];
	int total = playListPtr->getSongsByGenre( genre, similarSongs );
	
	for( int I = 0; I < total; I++ )
		cout << "'" << similarSongs[I]->getTitle() << "' by " << similarSongs[I]->getSinger() << endl;
}

//////////////////////////////// Class definitions for PlayList ////////////////////////////////////

// Constructor.
PlayList::PlayList()
{
	nSongs = 0;
	cout << "Playlist has been created, and it's empty" << endl;
}

// Destructor.
PlayList::~PlayList()
{
	for( int I = 0; I < nSongs; I++ )
		delete songs[I];
	cout << "Playlist has been destroyed." << endl;
}

// Add a song.
const Song* PlayList::addSong( const string title, const string genre, const string singer )
{
	if( nSongs < MAX_SONGS )
	{
		songs[nSongs] = new Song( title, genre, singer, this );
		nSongs++;
		
		return songs[nSongs-1];
	}
	
	cout << "No more songs can be added" << endl;
	return nullptr;
}

// Utility.
int PlayList::getSongsByGenre( const string genre, Song* list[] ) const
{
	int count = 0;
	
	for( int I = 0; I < nSongs; I++ )
	{
		if( songs[I]->getGenre() == genre )
		{
			list[count] = songs[I];
			count++;
		}
	}
	return count;
}

// Removing all songs that correspond to some artist.
int PlayList::removeSongsBySinger( const string singer )
{
	int deleted = 0;
	int I = 0;
	while( I < nSongs )
	{
		if( songs[I]->getSinger() == singer )
		{
			delete songs[I];
			songs[I] = songs[nSongs-1];
			nSongs--;
			deleted++;
		}
		else
			I++;
	}
	
	return deleted;
}

//////////////////////////////////////// Testing the classes ///////////////////////////////////////

int main()
{
	PlayList pl;
	pl.addSong( "I promised myself", "Pop", "ATeens" );
	pl.addSong( "In the end", "Rock", "Linkin Park" );
	const Song* s1 = pl.addSong( "What you waiting for?", "Pop", "Gwen Stefani" );
	pl.addSong( "Call me baby", "KPop", "EXO" );
	pl.addSong( "Upside down", "Pop", "ATeens" );
	
	s1->getSimilarSongs();
	
	int r = pl.removeSongsBySinger( "ATeens" );
	cout << "Removed: " << r << " songs!" << endl;
	
	s1->getSimilarSongs();
	
	return 0;
}





